#!/bin/sh
#
# cid.sh - apply CMap to CID-Keyed font
#
# Copyright (C) 2000,2001 Taiji Yamada <taiji@aihara.co.jp>
# All rights reserved.
# This software is provided under the BSD license. For full details,
# please see http://www.freebsd.org/copyright/license.html.
#
# [Acknowledgments]
# It is a pleasure to thank Hideyuki Suzuki <hideyuki@sat.t.u-tokyo.ac.jp>,
# Jerry Zhang <wenlongzhang@redflag-linux.com>,
# Chih-Wei Huang <cwhuang@linux.org.tw>, Yasuhiro Take <take@debian.org>,
# Toshiya Suzuki <mpsuzuki@hiroshima-u.ac.jp> and
# Masatake YAMATO <masata-y@is.aist-nara.ac.jp> for many precious suggestions
# and stimulating discussions about this work.
#
#ResourceDir=.
ResourceDir=`echo "pssystemparams /GenericResourceDir get =" | gsnd -q - `
FontDir=$ResourceDir/Font
CIDFontDir=$ResourceDir/CIDFont
# unnecessary since GS 6.01
#fonts_dir=../fonts
#link_from=../Resource/Font

[ -d $FontDir ] || mkdir $FontDir
[ -d $CIDFontDir ] || mkdir $CIDFontDir

case `basename $0` in
######################################################################
# Adobe CNS1
#
'ac13.sh')
FONTS='
MOESung-Regular
MOEKai-Regular
MHei-Medium-Acro
MSung-Light-Acro
'
[ -d $ResourceDir/ac13/CMap ] && CMAPS=`ls $ResourceDir/ac13/CMap/` || CMAPS=`cd $ResourceDir/CMap/; /bin/grep -l 'Adobe CNS1' *`
;;
######################################################################
# Adobe GB1
#
'ag13.sh')
FONTS='
STSong-Light-Acro
'
[ -d $ResourceDir/ag13/CMap ] && CMAPS=`ls $ResourceDir/ag13/CMap/` || CMAPS=`cd $ResourceDir/CMap/; /bin/grep -l 'Adobe GB1' *`
;;
######################################################################
# Adobe Japan1
#
'aj14.sh')
FONTS='
WadaGo-Bold
WadaMaruGo-Regular
WadaMin-Bold
WadaMin-Regular
HeiseiKakuGo-W5-Acro
HeiseiMin-W3-Acro
'
[ -d $ResourceDir/aj14/CMap ] && CMAPS=`ls $ResourceDir/aj14/CMap/` || CMAPS=`cd $ResourceDir/CMap/; /bin/grep -l 'Adobe Japan1' *`
;;
######################################################################
# Adobe Japan2
#
'aj20.sh')
FONTS='
WadaMaruGo-RegularH
WadaMin-RegularH
'
[ -d $ResourceDir/aj20/CMap ] && CMAPS=`ls $ResourceDir/aj20/CMap/` || CMAPS=`cd $ResourceDir/CMap/; /bin/grep -l 'Adobe Japan2' *`
;;
######################################################################
# Adobe Korea1
#
'ak12.sh')
FONTS='
Munhwa-Bold
Munhwa-Regular
MunhwaGothic-Bold
MunhwaGothic-Regular
MunhwaGungSeo-Bold
MunhwaGungSeo-Light
MunhwaGungSeoHeulim-Bold
MunhwaGungSeoHeulim-Light
MunhwaHoonMin-Regular
HYGoThic-Medium-Acro
HYSMyeongJo-Medium-Acro
'
[ -d $ResourceDir/ak12/CMap ] && CMAPS=`ls $ResourceDir/ak12/CMap/` || CMAPS=`cd $ResourceDir/CMap/; /bin/grep -l 'Adobe Korea1' *`
;;
*)
do_ordering_detection='yes'
;;
esac

[ -d $ResourceDir/ai0/CMap ] && CMAPS="$CMAPS "`ls $ResourceDir/ai0/CMap/` || CMAPS="$CMAPS "`cd $ResourceDir/CMap/; /bin/grep -l 'Adobe Identity' *`

case $1 in
'install')
  if [ "$2" != '' ]; then
    shift
    FONTS=$*
  fi
  for font in $FONTS; do
    if [ ! -f $CIDFontDir/$font ]; then
      echo "$font not found in $CIDFontDir"; continue
    fi
    if [ "$do_ordering_detection" = 'yes' ]; then
      Ordering=`/bin/sed -n '
/^.*\/Ordering (.*) def/{
  s/^.*\/Ordering (\(.*\)) def/\1/p
  q
}' $CIDFontDir/$font`
      case $Ordering in
      'CNS1')	##################################################
[ -d $ResourceDir/ac13/CMap ] && CMAPS=`ls $ResourceDir/ac13/CMap/` || CMAPS=`cd $ResourceDir/CMap/; /bin/grep -l 'Adobe CNS1' *`
        ;;
      'GB1')	##################################################
[ -d $ResourceDir/ag13/CMap ] && CMAPS=`ls $ResourceDir/ag13/CMap/` || CMAPS=`cd $ResourceDir/CMap/; /bin/grep -l 'Adobe GB1' *`
        ;;
      'Japan1')	##################################################
[ -d $ResourceDir/aj14/CMap ] && CMAPS=`ls $ResourceDir/aj14/CMap/` || CMAPS=`cd $ResourceDir/CMap/; /bin/grep -l 'Adobe Japan1' *`
        ;;
      'Japan2')	##################################################
[ -d $ResourceDir/aj20/CMap ] && CMAPS=`ls $ResourceDir/aj20/CMap/` || CMAPS=`cd $ResourceDir/CMap/; /bin/grep -l 'Adobe Japan2' *`
        ;;
      'Korea1')	##################################################
[ -d $ResourceDir/ak12/CMap ] && CMAPS=`ls $ResourceDir/ak12/CMap/` || CMAPS=`cd $ResourceDir/CMap/; /bin/grep -l 'Adobe Korea1' *`
        ;;
      *)
        echo "$font can't detect Ordering"; continue
        ;;
      esac
[ -d $ResourceDir/ai0/CMap ] && CMAPS="$CMAPS "`ls $ResourceDir/ai0/CMap/` || CMAPS="$CMAPS "`cd $ResourceDir/CMap/; /bin/grep -l 'Adobe Identity' *`
    fi
    for cmap in $CMAPS; do
      case $cmap in
      'Roman'|'Katakana'|'Hiragana'|'Hankaku')
        to='.'
        ;;
      *-UCS2|*-UCS2C)
        continue
        ;;
      *)
        to='-'
        ;;
      esac
      [ ! -f $FontDir/$font$to$cmap ] && cat <<EOF > $FontDir/$font$to$cmap && echo "+$font$to$cmap"
%!PS-Adobe-3.0 Resource-Font
%%DocumentNeededResources: $cmap (CMap)
%%IncludeResource: $cmap (CMap)
%%BeginResource: Font ($font$to$cmap)
/$font$to$cmap
/$cmap /CMap findresource
[/$font /CIDFont findresource]
composefont pop
%%EndResource
%%EOF
EOF
      # unnecessary since GS 6.01
      #[ ! -f $fonts_dir/$font$to$cmap ] && (cd $fonts_dir; ln -s $link_from/$font$to$cmap .)
    done
  done
  ;;
'uninstall')
  if [ "$2" != '' ]; then
    shift
    FONTS=$*
  fi
  if [ "$do_ordering_detection" = 'yes' ]; then
    CMAPS=`cd $ResourceDir/CMap/; /bin/grep -l 'Adobe' *`
  fi
  for font in $FONTS; do
    for cmap in $CMAPS; do
      case $cmap in
      'Roman'|'Katakana'|'Hiragana'|'Hankaku')
        to='.'
        ;;
      *-UCS2|*-UCS2C)
        continue
        ;;
      *)
        to='-'
        ;;
      esac
      # unnecessary since GS 6.01
      #[ -f $fonts_dir/$font$to$cmap ] && rm -f $fonts_dir/$font$to$cmap
      [ -f $FontDir/$font$to$cmap ] && rm -f $FontDir/$font$to$cmap && echo "-$font$to$cmap"
    done
  done
  ;;
*)
  cat <<EOF
usage: $0 'install|uninstall' [Font...]
please edit the lines FONTS=... of me, before use without args.
EOF
  ;;
esac
