#!/usr/bin/perl -s
#
# icmaptops.pl - writes inverse mapping of CMap to PostScript file
#
# usage:
#	At first, please edit $in_dir and $out_dir.
#	$in_dir has ac13/, ag13/, aj14/, aj20/ and ak12/ directories.
#	In $out_dir, TrueType/* resources will be created.
#	% emacs icmaptops.pl
#	% icmaptops.pl
#
# Copyright (C) 2000/11/15-23,12/3,13 Taiji Yamada <taiji@aihara.co.jp>
# All rights reserved.
# This software is provided under the BSD license. For full details,
# please see http://www.freebsd.org/copyright/license.html.
#
my $creator = 'Taiji Yamada <taiji@aihara.co.jp>';

#my $in_dir = '/usr/local/share/ghostscript/Resource';
#my $out_dir = 'TrueType-1999';
my $in_dir = '.';
my $out_dir = 'TrueType-';

( ! -d $out_dir ) && mkdir $out_dir, 0755;

my $cid2code_1999 = {
  'Adobe-CNS1'   => "$in_dir/ac10/cid2code.txt",
  'Adobe-GB1'    => "$in_dir/ag12/cid2code.txt",
  'Adobe-Japan1' => "$in_dir/aj12/cid2code.txt",
  'Adobe-Japan2' => "$in_dir/aj20/cid2code.txt",
  'Adobe-Korea1' => "$in_dir/ak11/cid2code.txt",
};
my $cid2code_2000 = {
  'Adobe-CNS1'   => "$in_dir/ac13/cid2code.txt",
  'Adobe-GB1'    => "$in_dir/ag13/cid2code.txt",
  'Adobe-Japan1' => "$in_dir/aj14/cid2code.txt",
  'Adobe-Japan2' => "$in_dir/aj20/cid2code.txt",
  'Adobe-Korea1' => "$in_dir/ak12/cid2code.txt",
};
#my $cid2code = $cid2code_1999;
my $cid2code = $cid2code_2000;

my $cid2ttf = {
  'CNS1,Big5'       => { base=>'ETen-B5',      sup=>0, },
  'CNS1,Unicode'    => { base=>'UniCNS-UCS2',  sup=>0, },
  'GB1,PRC'         => { base=>'GBK-EUC',      sup=>1, },
  'GB1,Unicode'     => { base=>'UniGB-UCS2',   sup=>1, },
  'Japan1,ShiftJIS' => { base=>'90ms-RKSJ',    sup=>2, },
  'Japan1,Unicode'  => { base=>'UniJIS-UCS2',  sup=>4, },
  'Japan2,Unicode'  => { base=>'UniHojo-UCS2', sup=>0, },
  'Korea1,Wansung'  => { base=>'KSCms-UHC',    sup=>1, },
  'Korea1,Johab'    => { base=>'KSC-Johab',    sup=>1, },
  'Korea1,Unicode'  => { base=>'UniKS-UCS2',   sup=>1, },
};

for my $imap (sort keys %$cid2ttf) {
  my($reg, $ord, $sup) = ('Adobe', $imap =~ /(.*),.*/, $cid2ttf->{$imap}->{sup});
  my($reg_ord, $code) = ("$reg-$ord", $imap =~ /.*,(.*)/);
  my $base = $cid2ttf->{$imap}->{base};
  open(IN, "$cid2code->{$reg_ord}") || die "$!";
  open(DEBUG, ">$cid2code->{$reg_ord}_$base") || die "$!" if ($DEBUG);
  print("$out_dir/$reg_ord-$code\n");
  open(OUT, ">$out_dir/$reg_ord-$code") || die "$!";
  my($fi) = undef;
  my($n, %cm, %dcm, %vcm, %vdcm) = (0, (), (), (), ());
  while (<IN>) {
    chomp;
    if (/^#/) {
      next;
    }
    elsif (!$fi && /^CID(\t.*)+/) {
      my @f = split(/\t+/, $_);
      for (my $i=0; $i<@f; $i++) {
	$fi = $i, last if ($f[$i] eq $base);
      }
      ($fi) || die "error: not found $base";
      printf DEBUG ("CID\t%s\n", $f[$fi]) if ($DEBUG);
    }
    elsif ($fi && /^\d+\t/) {
      my @f = split(/\t+/, $_);
      my $id = $f[0] + 0;
      my(@h, @v) = ((), ());
      ($f[$fi] eq '*') && printf DEBUG ("%d\t%s\n", $id, '*') if ($DEBUG);
      if ($f[$fi] eq '*') {
	next;
      }
      else {
	my @c = split(/,/, $f[$fi]);
	#printf DEBUG ("%d\t%s\n", $id, $f[$fi]) if ($DEBUG);
	foreach my $c (@c) {
	  if ($c =~ /(.*)v$/) { 
	    push(@v, hex($1));
	  }
	  else {
	    push(@h, hex($c));
	  }
	}
	@h = sort numerically @h; # doubly sure
	@v = sort numerically @v; # doubly sure
      }
      $cm{$id} = $h[0] if (@h > 0);
      $dcm{$id} = [@h] if (@h > 1);
      $vcm{$id} = $v[0] if (@v > 0);
      $vdcm{$id} = [@v] if (@v > 1);
      if ($code =~ /Unicode/) {
	printf DEBUG ("%d\t%s\n", $id,
		      join(',', map(sprintf("%0*x", 4, $_),@h),
			   map(sprintf("%0*xv", 4, $_),@v))) if ($DEBUG);
      }
      else {
	printf DEBUG ("%d\t%s\n", $id,
		      join(',', map(sprintf("%0*x", ($_<=0xff)?2:4, $_),@h),
			   map(sprintf("%0*xv", ($_<=0xff)?2:4, $_),@v))) if ($DEBUG);
      }
      $n = $id;
    }
  }
  close(DEBUG) if ($DEBUG);
  close(IN);
  my $id;
  print OUT<<EOF;
%!PS-Adobe-3.0 Resource-TrueType
%%Creator: $creator
%%BeginResource: TrueType ($reg_ord-$code)
7 dict begin
/Registry ($reg) def
/Ordering ($ord) def
/Supplement $sup def
/Code [
EOF
  #
  # Code
  #
  for ($id = 0; $id <= $n; $id++) {
    my $c = $cm{$id};
    if ($cm{$id} == 0) { 
      # mix vertical glyphs
      $c = $vcm{$id};
    }
    elsif ($code =~ /Unicode/ && $vcm{$id} != 0 &&
	   # mix vertical glyphs in R (Restrict) region
	   # except for full-width latin and half-width katakana
	   0xf900 <= $cm{$id} && !(0xff00 <= $cm{$id} && $cm{$id} <= 0xff9f)) {
      $c = $vcm{$id};
    }
    printf OUT ("%s16#%04x%s",
		(($id % 8 == 0 && $id) ? "\n" : ($id ? ' ': '')),
		$c, ($id == $n) ? "\n" : "");
  }
  #
  print OUT<<EOF;
] def
/DuplicateCodeMap [
EOF
  #
  # DuplicateCodeMap
  #
  for $id (sort numerically keys %dcm) {
    printf OUT ("[ %d\t", $id);
    foreach $c (sort numerically @{$dcm{$id}}) {
      printf OUT ("16#%04x ", $c);
    }
    printf OUT ("]\n");
  }
  #
  print OUT<<EOF;
] def
/VerticalCodeMap [
EOF
  #
  # VerticalCodeMap
  #
  for $id (sort numerically keys %vcm) {
    if ($cm{$id} == 0) {
      # mix vertical glyphs
      ;
    }
    elsif ($code =~ /Unicode/ && $vcm{$id} != 0 &&
	   # mix vertical glyphs in R (Restrict) region
	   # except for full-width latin and half-width katakana
	   0xf900 <= $cm{$id} && !(0xff00 <= $cm{$id} && $cm{$id} <= 0xff9f)) {
      ;
    }
    else {
      print OUT '%'; # do not mix vertical glyphs, otherwise
    }
    printf OUT ("[ %d\t16#%04x ]\n", $id, $vcm{$id});
  }
  #
  print OUT<<EOF;
] def
/VerticalDuplicateCodeMap [
EOF
  #
  # VerticalDuplicateCodeMap
  #
  for $id (sort numerically keys %vdcm) {
    printf OUT ("[ %d\t", $id);
    foreach $c (sort numerically @{$vdcm{$id}}) {
      printf OUT ("16#%04x ", $c);
    }
    printf OUT ("]\n");
  }
  #
  print OUT<<EOF;
] def
currentdict end /$reg_ord-$code exch /TrueType defineresource pop
%%EndResource
%%EOF
EOF
  close(OUT);
}

sub numerically {
  $a <=> $b;
}
